/* $Id: net.c,v 1.41 2003/04/10 13:22:52 blues Exp $ */

/* Intro {{{
 * ---------------------------------------------------------------- 
 *  DConnect Daemon
 * 
 *      #(@) Copyright (c) 2002, DConnect development team
 *      #(@) Homepage: http://www.dc.ds.pg.gda.pl/
 *
 *  This  program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You  should  have  received a copy  of  the GNU General Public
 *  License  along  with  this  program;  if  not,  write  to  the
 *  Free  Software  Foundation,  Inc.,  675  Mass  Ave,  Cambridge,
 *  MA 02139, USA. 
 *
 * ---------------------------------------------------------------- 
 * }}} */

/* include local header files {{{ */
#include "pch.h"
#include "main.h"
#include "misc.h"
#include "console.h"
/* }}} */

extern struct userrec *user[];
extern int NUSERS;

/* sendf() - something like printf(), but to a socket {{{ */
void sendf( struct userrec * usr, const char * fmt, ... )
{
  int n;
  char *str;
  va_list args;
  va_start( args, fmt );
  str=(char *)make_message(fmt,args);
  va_end( args );

  if (usr) send( usr->sock, str, strlen( str ), 0 );
  else for ( n = 0; n < NUSERS; n++ )
	   /* multicasting only to non-console users */
           if ((user[n]->state>1)&&(user[n]->cons==0)) send( user[n]->sock, str, strlen( str ), 0 );
  my_free(str);
} /* }}} */

/* this is required, since after disconnect a socket is switched to a TIME_WAIT
 * state and the listening port remains occupied for several minutes */
/* setsockopts() {{{ */
char setsockopts(int *sock,unsigned char reuse) {
	int flag=1;
	char ret=0;
	struct linger linger;

	if (reuse) {
	    linger.l_onoff=0; linger.l_linger=0;
	    if (setsockopt(*sock,SOL_SOCKET,SO_REUSEADDR,&flag,sizeof(flag))<0) ret=1;
	    if (setsockopt(*sock,SOL_SOCKET,SO_LINGER,&linger,sizeof(linger))<0) ret=1;
	}
	if (setsockopt(*sock,SOL_SOCKET,SO_KEEPALIVE,&flag,sizeof(flag))<0) ret=1;
	return ret;
} /* }}} */

/* publicmsg() - send message as "Hub" to public chat {{{ */
void publicmsg(struct userrec *to,char *msg)
{
	if (to) if (to->cons) { sendf(to,"%s\r\n",msg); return; }
	sendf(to,"<Hub> %s|",msg);
} /* }}} */

/* privmsg() - send message as "Hub" to private chat {{{ */
void privmsg(struct userrec *to,char *msg)
{
	if (!to) return;
	if (to->state<2) return;
	if (to->cons) publicmsg(to,msg);
	else sendf(to,"$To: %s From: Hub $<Hub> %s|",to->nick,msg);
} /* }}} */

/* disconnect() - disconnect a user with optional reason message {{{ */
void disconnect(struct userrec *usr,char *reason) {
	if (reason) publicmsg(usr,reason);
	close(usr->sock);
	debug(DEBUG_NET,"Disconnected %s@%s: %s",(usr->nick)?usr->nick:"Unknown",usr->ip,(reason)?reason:"Connection closed.");
	my_free(usr->buf);
	my_free(usr->nick);
	my_free(usr->ip);
	my_free(usr->ver);
	my_free(usr->desc);
	my_free(usr->ctype);
	my_free(usr->email);
	my_free(usr->perm);
	my_free(usr);
} /* }}} */

/* bind_test() - test for permission to bind to a port {{{ */
void bind_test(unsigned short int listen_port) {
	struct sockaddr_in local;
	int test_sock;
	if ((test_sock=socket(AF_INET,SOCK_STREAM,0))==-1) {
		debug(DEBUG_STD,"Socket error - terminating: %s.",strerror(errno));
		exit(0);
	}
	if (setsockopts(&test_sock,1))
		debug(DEBUG_STD,"setsockopts() failed.");
	bzero(&local,sizeof(struct sockaddr_in));
	local.sin_family=AF_INET;
	local.sin_port=htons(listen_port);
	local.sin_addr.s_addr=INADDR_ANY;
	if (bind(test_sock,(struct sockaddr *)&local,sizeof(struct sockaddr))<0) {
		debug(DEBUG_STD,"Can't bind to port %d - terminating: %s.",listen_port,strerror(errno));
		exit(0);
	}
	close(test_sock);
} /* }}} */

/* strip_telnet() - strip telnet codes - from eggdrop {{{ */
void strip_telnet(struct userrec *usr,char *buf,int *len) {
  unsigned char *p = (unsigned char *) buf, *o = (unsigned char *) buf;
  int mark;

  while (*p != 0) {
    while ((*p != TLN_IAC) && (*p != 0))
      *o++ = *p++;
    if (*p == TLN_IAC) {
      p++;
      mark = 2;
      if (!*p)
	mark = 1;		/* bogus */
      if ((*p >= TLN_WILL) && (*p <= TLN_DONT)) {
	mark = 3;
	if (!*(p + 1))
	  mark = 2;		/* bogus */
      }
      if (*p == TLN_WILL) {
	/* WILL X -> response: DONT X */
	/* except WILL ECHO which we just smile and ignore */
	if (*(p + 1) != TLN_ECHO) {
	  write(usr->sock, TLN_IAC_C TLN_DONT_C, 2);
	  write(usr->sock, p + 1, 1);
	}
      }
      if (*p == TLN_DO) {
	/* DO X -> response: WONT X */
	/* except DO ECHO which we just smile and ignore */
	if (*(p + 1) != TLN_ECHO) {
	  write(usr->sock, TLN_IAC_C TLN_WONT_C, 2);
	  write(usr->sock, p + 1, 1);
	}
      }
      if (*p == TLN_AYT) {
	/* "are you there?" */
	/* response is: "hell yes!" */
	write(usr->sock, "\r\nHell, yes!\r\n", 14);
      }
      /* Anything else can probably be ignored */
      p += mark - 1;
      *len = *len - mark;
    }
  }
  *o = *p;
} /* }}} */

/* VIM Settings {{{
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * soft-stop-width: 4
 * c indent on
 * End:
 * vim600: sw=4 ts=4 sts=4 cindent fdm=marker
 * vim<600: sw=4 ts=4
 * }}} */

